#define RESVG_CAIRO_BACKEND

#include <cairo.h>
#include <resvg.h>
#include <resvg_port.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_INTERNAL_TRUETYPE_TYPES_H

  /* global variable to hold state, we don't have objects in C */
  Resvg_Port_State  state;

  FT_Error
  resvg_port_init(  )
  {
    resvg_init_log();
    resvg_init_options(&state.opt);
    state.opt.font_family       = "Times New Roman";
    state.opt.languages         = "en";
    state.opt.keep_named_groups = true;
    return FT_Err_Ok; 
  }

  FT_Error
  resvg_port_free(  )
  {
    return FT_Err_Ok;
  }

  FT_Error
  resvg_port_render( FT_GlyphSlot slot_ )
  {
    int           error;
    TT_GlyphSlot  slot;
    unsigned int  glyph_index;
    resvg_rect    bbox;
    resvg_size    size;
    FT_Face       face;
    FT_Size       ft_size;

    unsigned char*  buffer;
    int             width;
    int             height;
    int             bitmap_left;
    int             bitmap_top;
    int             pitch;

    slot  = (TT_GlyphSlot)slot_;

    /* Get the size object */
    face    = slot_->face;
    ft_size = face->size;

    error = resvg_parse_tree_from_data( (const char*)slot->svg_document,
                                      (const size_t)slot->svg_document_length,
                                      &state.opt,
                                      &state.tree );
    /* TODO: (OT-SVG) Do better error handling. */
    if(error != RESVG_OK)
      return -1;
    
    glyph_index = slot_->glyph_index;

    /* TODO: (OT-SVG) For now supposing that only one glyph exists */
    size  = resvg_get_image_size( state.tree );
    float  x_scale = (float)ft_size->metrics.x_ppem /(float)size.width;
    float  y_scale = (float)ft_size->metrics.y_ppem /(float)size.height;
    error = resvg_get_image_bbox( state.tree, &bbox);

    state.surface = cairo_image_surface_create( CAIRO_FORMAT_ARGB32, 
                                               bbox.width*x_scale, bbox.height*y_scale );
    state.cr      = cairo_create(state.surface); 

    /* before doing the rendering perform a cairo translate */
    cairo_scale    (state.cr, x_scale, y_scale);
    cairo_translate(state.cr, -1*bbox.x, (-1)*bbox.y);

    /* do the actual rendering */
    resvg_cairo_render_to_canvas( state.tree, 
                                  &state.opt, 
                                  size, state.cr );
    
    /* cairo_surface_write_to_png( state.surface, "output.png" ); */

    /* get the data out manually so we can put it in buffer */
    buffer = cairo_image_surface_get_data( state.surface );
    width  = cairo_image_surface_get_width( state.surface );
    height = cairo_image_surface_get_height( state.surface );
    pitch  = cairo_image_surface_get_stride( state.surface );   
  
    slot->root.bitmap.buffer = buffer;
    slot->root.bitmap.rows   = height;
    slot->root.bitmap.width  = width;
    slot->root.bitmap.pitch  = pitch;

    int total_bytes = height * pitch;
    for( int i = 0; i < total_bytes; i+= 4 )
    {
      unsigned char  a, r, g, b;
      b = buffer[i];
      g = buffer[i + 1];
      r = buffer[i + 2];
      a = buffer[i + 3];
      
      buffer[i]     = b; /* b */
      buffer[i + 1] = g; /* g */
      buffer[i + 2] = r; /* r */
      buffer[i + 3] = a;
    }

    return FT_Err_Ok;
  }
